﻿namespace anbs_cp
{
    /// <summary>
    /// Форматирует размерности в понятную строку
    /// </summary>
    public interface IValueFormatter
    {

        #region Определения интерфейса
        /// <summary>
        /// Имена размерностей
        /// </summary>
        public string[] ValueNames { get; set; }
        /// <summary>
        /// Знаков после запятой
        /// </summary>
        public byte DecimalPlaces { get; set; }
        /// <summary>
        /// Максимальные размеры (массив ulong[4])
        /// </summary>
        public long[] MaxSizes { get; set; }
        #endregion

        #region Методы интерфейса
        /// <summary>
        /// Форматирование размерности
        /// </summary>
        /// <param name="value">Размерность, требующая форматирования</param>
        /// <returns>Форматированная размерность (например, 20 Мб)</returns>
        public string Format(long value)
        {
            //Bytes
            if (value < MaxSizes[0])
                return $"{value} {ValueNames[0]}";
            //KiloBytes
            if ((value >= MaxSizes[0]) && (value < MaxSizes[1]))
                return $"{FrmtSize(value, MaxSizes[0])} {ValueNames[1]}";
            //MegaBytes
            if ((value >= MaxSizes[1]) && (value < MaxSizes[2]))
                return $"{FrmtSize(value, MaxSizes[1])} {ValueNames[2]}";
            //GigaBytes
            if ((value >= MaxSizes[2]) && (value < MaxSizes[3]))
                return $"{FrmtSize(value, MaxSizes[2])} {ValueNames[3]}";
            //TeraBytes
            if (value >= MaxSizes[3])
                return $"{FrmtSize(value, MaxSizes[3])} {ValueNames[4]}";
            //Не определён
            return $"{value} {ValueNames[0]}";
        }
        /// <summary>
        /// Деление числа на число с DecimalPlaces знаками после запятой
        /// </summary>
        /// <param name="dividend">Делимое число</param>
        /// <param name="divider">Число-делитель</param>
        /// <returns>Частное (с DecimalPlaces знаками после запятой)</returns>
        private string FrmtSize(long dividend, long divider)
        {
            long delim = 1;

            for (int i = 0; i <= DecimalPlaces; i++)
            {
                delim *= 10;
            }

            decimal value = Math.Round((decimal)(dividend * delim / divider)) / delim;

            return $"{value}";
        }
        #endregion
    }
}